︠8e31e2e2-9aab-456b-b7f8-fd8ef85fb628s︠
%auto
typeset_mode(True)
︡596a2909-8502-4884-838e-1f9c2189b6aa︡{"done":true}︡
︠1ade7f4c-5bc8-4d29-b224-9b6fdf503adcs︠
# Defining the phase space variables
var('x, v, t')
︡a45982ce-73d3-433d-8d82-77ae8e4ab81b︡{"html":"<div align='center'>($\\displaystyle x$, $\\displaystyle v$, $\\displaystyle t$)</div>"}︡{"done":true}︡
︠faa24ed1-eba1-4000-b660-30f97df99f66s︠
# Defining the constants
var('k, m')
︡8a8a728b-751f-4b6d-a329-b3268d862b76︡{"html":"<div align='center'>($\\displaystyle k$, $\\displaystyle m$)</div>"}︡{"done":true}︡
︠525b48ac-b8df-432c-ae6b-a13bd3dbb454s︠
# Definitions of the functions that vary with space and time:
var('v_b, T, rho')
︡cdff31a8-8101-427c-9df4-1056d69ff19c︡{"html":"<div align='center'>($\\displaystyle v_{b}$, $\\displaystyle T$, $\\displaystyle \\rho$)</div>"}︡{"done":true}︡
︠a429b662-ab5c-4d50-ba2a-e24213336a9as︠
# Defining collision time-scale
var('tau')
︡1a5ea300-c346-46d2-a186-06a51e5431ed︡{"html":"<div align='center'>$\\displaystyle \\tau$</div>"}︡{"done":true}︡
︠30680b4a-b5b4-4176-aa30-c63a4481f40bs︠
# Definition of the distribution function
distribution_function_f_0 = rho * (sqrt(m/(2*pi*k*T))) * exp(-m*(v - v_b)^2/(2*k*T))
distribution_function_f_0
︡52104c66-8026-4557-8bd4-85598bba198c︡{"html":"<div align='center'>$\\displaystyle \\sqrt{\\frac{1}{2}} \\rho \\sqrt{\\frac{m}{\\pi T k}} e^{\\left(-\\frac{m {\\left(v - v_{b}\\right)}^{2}}{2 \\, T k}\\right)}$</div>"}︡{"done":true}︡
︠5741b061-039e-4ed6-83b1-688851060d19s︠
# Declaring the positive quantities:
assume(m>0)
assume(k>0)
assume(T>0)
assume(rho>0)
︡52d4f785-2f82-4944-9d78-e59720fd6125︡{"done":true}︡
︠33e02f04-4fc1-43b5-88bc-3dc3119af4d8s︠
# Redefining the variables that vary with space and time:
v_bulk = function('v_{bulk}')(x, t)
T      = function('T')(x, t)
rho    = function('rho')(x, t)
︡9cc7abbf-89e6-45ef-abfd-61d8dbaa9228︡{"done":true}︡
︠e9478e78-a933-4f7e-a60e-639f3c973cc3s︠
# Defining the various velocity moments:
zeroth_moment = integrate(distribution_function_f_0,      v, -infinity, infinity).subs(T=T, rho=rho, v_b=v_bulk)
first_moment  = integrate(distribution_function_f_0*v,    v, -infinity, infinity).subs(T=T, rho=rho, v_b=v_bulk)
second_moment = integrate(distribution_function_f_0*v**2, v, -infinity, infinity).subs(T=T, rho=rho, v_b=v_bulk)
third_moment  = integrate(distribution_function_f_0*v**3, v, -infinity, infinity).subs(T=T, rho=rho, v_b=v_bulk)
fourth_moment = integrate(distribution_function_f_0*v**4, v, -infinity, infinity).subs(T=T, rho=rho, v_b=v_bulk)
︡05df8c29-ffa9-4203-81cc-878a4b019845︡{"done":true}︡
︠27619968-cb33-450d-9cc2-87a0d3bad33as︠
# Simplifying the expressions
zeroth_moment = zeroth_moment.expand().simplify()
first_moment  = first_moment.expand()
second_moment = second_moment.expand()
third_moment  = third_moment.expand()
fourth_moment = fourth_moment.expand()
︡5e7a0ebf-639b-401c-85d7-32390c8ffdb7︡{"done":true}︡
︠40f3b73d-3651-4f62-a32d-f1b819449518s︠
# Obtaining the Euler equations
euler_continuity_eqn = zeroth_moment.diff(t) + first_moment.diff(x)
euler_momentum_eqn   = first_moment.diff(t)  + second_moment.diff(x)
euler_energy_eqn     = second_moment.diff(t) + third_moment.diff(x)
︡c39039f4-513f-451b-90ba-d645e3dbd560︡{"done":true}︡
︠6539b809-bcc3-4288-97e6-3a9291ad6107s︠
# Obtaining the NS equations

# The expression for the distribution function f_approx is f = f_0 - tau * v * df_0/dx
# Due to mass, momentum and energy conservation:
# \int     f dv = \int     f_0 dv
# \int v   f dv = \int v   f_0 dv
# \int v^2 f dv = \int v^2 f_0 dv

NS_continuity_eqn = zeroth_moment.diff(t) + first_moment.diff(x)
NS_momentum_eqn   = first_moment.diff(t) + second_moment.diff(x)
NS_energy_eqn     = second_moment.diff(t)  + (third_moment - tau * fourth_moment.diff(x)).diff(x)
︡ce6b8abd-ab7e-426a-bbf9-55d7f048a832︡{"done":true}︡
︠3cfcd40b-252c-4ce1-8806-1c4874fcefa4s︠
# Linearizing the system:
var('rho0, delta_rho, T0, delta_T, v0, delta_v, I, k1, delta_rho_dt, delta_T_dt, delta_v_dt')
︡114be8b0-1c0c-4e2d-b6da-9922b4b796f2︡{"html":"<div align='center'>($\\displaystyle \\rho_{0}$, $\\displaystyle \\delta_{\\rho}$, $\\displaystyle T_{0}$, $\\displaystyle \\delta_{T}$, $\\displaystyle v_{0}$, $\\displaystyle \\delta_{v}$, $\\displaystyle I$, $\\displaystyle k_{1}$, $\\displaystyle \\delta_{\\rho_{\\mathit{dt}}}$, $\\displaystyle \\delta_{T_{\\mathit{dt}}}$, $\\displaystyle \\delta_{v_{\\mathit{dt}}}$)</div>"}︡{"done":true}︡
︠81db5793-8d3c-4acc-9552-78bef6ba84f0s︠
def linearize(term):
    return taylor(term, (delta_rho, 0), \
                        (delta_v, 0),  \
                        (delta_T, 0),   \
                        (delta_rho_dt, 0), \
                        (delta_v_dt, 0),  \
                        (delta_T_dt, 0), 1 \
                 ).simplify_full()

eqn_continuity = linearize(NS_continuity_eqn.subs(rho==rho0+delta_rho, rho.diff(x)==I*k1*delta_rho, rho.diff(t)==delta_rho_dt, \
                                                  T==T0+delta_T, T.diff(x)==I*k1*delta_T, T.diff(t)==delta_T_dt, \
                                                  v_bulk==v0+delta_v, v_bulk.diff(x)==I*k1*delta_v, v_bulk.diff(t)==delta_v_dt
                                                 )
                          )
eqn_momentum   = linearize(NS_momentum_eqn.subs(rho==rho0+delta_rho, rho.diff(x)==I*k1*delta_rho, rho.diff(t)==delta_rho_dt, \
                                                T==T0+delta_T, T.diff(x)==I*k1*delta_T, T.diff(t)==delta_T_dt, \
                                                v_bulk==v0+delta_v, v_bulk.diff(x)==I*k1*delta_v, v_bulk.diff(t)==delta_v_dt
                                               )
                          )
eqn_energy     = linearize(NS_energy_eqn.subs(rho==rho0+delta_rho, rho.diff(x)==I*k1*delta_rho, rho.diff(t)==delta_rho_dt, rho.diff(x, 2)==-k1**2*delta_rho,\
                                              T==T0+delta_T, T.diff(x)==I*k1*delta_T, T.diff(t)==delta_T_dt, T.diff(x, 2)==-k1**2*delta_T,\
                                              v_bulk==v0+delta_v, v_bulk.diff(x)==I*k1*delta_v, v_bulk.diff(t)==delta_v_dt, v_bulk.diff(x, 2)==-k1**2*delta_v
                                             )
                          )

eqns          = [eqn_continuity == 0, eqn_momentum == 0, eqn_energy == 0]
delta_vars    = [delta_rho, delta_v, delta_T]
delta_vars_dt = [delta_rho_dt, delta_v_dt, delta_T_dt]

solutions = solve(eqns, delta_vars_dt, solution_dict=True)

solns_delta_vars_dt = []
for dvar_dt in delta_vars_dt:
    solns_delta_vars_dt.append(solutions[0][dvar_dt])

M = jacobian(solns_delta_vars_dt, delta_vars)
M = M.apply_map(lambda x : x.simplify_full())

pretty_print("Linearized system : ")
print("\n")
pretty_print(Matrix(delta_vars_dt).transpose(), " = ", M, Matrix(delta_vars).transpose())

︡15ce9147-ffb2-4aa8-9940-796b497452d1︡{"html":"<div align='center'>Linearized system : </div>"}︡{"stdout":"\n\n"}︡{"html":"<div align='center'>$\\displaystyle \\left(\\begin{array}{r}\n\\delta_{\\rho_{\\mathit{dt}}} \\\\\n\\delta_{v_{\\mathit{dt}}} \\\\\n\\delta_{T_{\\mathit{dt}}}\n\\end{array}\\right)$  =  $\\displaystyle \\left(\\begin{array}{rrr}\n-I k_{1} v_{0} &amp; -I k_{1} \\rho_{0} &amp; 0 \\\\\n-\\frac{I T_{0} k k_{1}}{m \\rho_{0}} &amp; -I k_{1} v_{0} &amp; -\\frac{I k k_{1}}{m} \\\\\n-\\frac{k_{1}^{2} m^{2} \\tau v_{0}^{4} + 6 \\, T_{0} k k_{1}^{2} m \\tau v_{0}^{2} + 3 \\, T_{0}^{2} k^{2} k_{1}^{2} \\tau}{k m \\rho_{0}} &amp; -\\frac{2 \\, {\\left(2 \\, k_{1}^{2} m \\tau v_{0}^{3} + 6 \\, T_{0} k k_{1}^{2} \\tau v_{0} + I T_{0} k k_{1}\\right)}}{k} &amp; -\\frac{6 \\, k_{1}^{2} m \\tau v_{0}^{2} + 6 \\, T_{0} k k_{1}^{2} \\tau + I k_{1} m v_{0}}{m}\n\\end{array}\\right)$ $\\displaystyle \\left(\\begin{array}{r}\n\\delta_{\\rho} \\\\\n\\delta_{v} \\\\\n\\delta_{T}\n\\end{array}\\right)$</div>"}︡{"done":true}︡
︠38ae2e69-ff4a-4f11-a346-6a55180740acr︠
rho0_num = 1
v0_num   = 0
T0_num   = 1
k1_num   = 2 * pi
tau_num  = 0.01

M_numerical = M.subs(rho0=rho0_num, v0=v0_num, T0=T0_num, k1=k1_num, tau = tau_num)

M_numerical = M_numerical.change_ring(CDF)
eigenvecs   = M_numerical.eigenvectors_right()


︡87af0157-4b5f-473b-a971-a59da6dcc9ba︡
︠754f1130-e2cc-4b50-b697-ad91aa2b3064︠









